package com.lstznkj.groundstation.controller;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.lstznkj.groundstation.DTO.ui.DataSourceRequestDTO;
import com.lstznkj.groundstation.DTO.ui.DataSourceUIDTO;
import com.lstznkj.groundstation.model.crane.DataSource;
import com.lstznkj.groundstation.service.crane.DataSourceService;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

@RestController
public class DataSourceController {
    @Autowired
    private DataSourceService dataSourceService;
    @Autowired
    private ObjectMapper objectMapper;
    private Logger logger = LoggerFactory.getLogger(getClass());


    @RequestMapping(value = "/dataSources",method = RequestMethod.POST)
    public Object add(@RequestBody DataSource dataSource){
        try {
            logger.info("start to add dataSource:"+objectMapper.writeValueAsString(dataSource));

        }
        catch (Exception ex){

        }
        return dataSourceService.add(dataSource);
    }



    @RequestMapping(value = "/dataSources/{id}",method = RequestMethod.PUT)
    public Object update(@PathVariable int id, @RequestBody DataSourceUIDTO dataSourceUIDTO){
        try {
            logger.info("start to update dataSource:"+objectMapper.writeValueAsString(dataSourceUIDTO));

        }
        catch (Exception ex){

        }
        return dataSourceService.update(dataSourceUIDTO);
    }

    /***
     * 获取指定id的设备的所有数据源，包括子设备
     * @param id
     * @return
     */
    @RequestMapping(value = "/devices/{id}/dataSources",method = RequestMethod.GET)
    public Object getAllDataSourcesByDeviceId(@PathVariable int id){
        try {
            logger.info("start to get device:"+id+" dataSources:");
            return  dataSourceService.getAllDataSources(id);
           //return   dataSourceService.getAllDataSourcesByDeviceId(id);
        }
        catch (Exception ex){
            ex.printStackTrace();
        }
        return null;
    }



    @RequestMapping(value = "/dataSources",method = RequestMethod.GET)
    public Object getAll(){

        return dataSourceService.getAll();
    }

    /**
     * 查询指定id的数据源数据（限定时间段）
     * @param id
     * @param startTime
     * @param endTime
     * @return
     */
    @RequestMapping(value = "/chartDataDTO/{id}/{startTime}/{endTime}",method = RequestMethod.GET)
    public Object getDataSourcesByIdAndDate(@PathVariable int id,@PathVariable String startTime,@PathVariable String endTime){

        return dataSourceService.getChartDataDTO(id,startTime,endTime);

    }

    /**
     * 查询图表数据，根据数据类型分组
     * @param startTime
     * @param endTime
     * @param param
     * @return
     */
    @RequestMapping(value = "/chartDataDTOList/{startTime}/{endTime}",method = RequestMethod.GET)
    public Object getChartDataDTOListByDate(@PathVariable String startTime, @PathVariable String endTime,String param ){
        List<DataSourceRequestDTO> dataSourceRequestDTOList=new ArrayList<>();
        //@RequestBody List<DataSourceRequestDTO> dataSourceRequestDTOList
        try {
            String[] idList=param.split(",");


            for(String s:idList){
                DataSourceRequestDTO d=new DataSourceRequestDTO();
                if(!s.equals("")){
                    d.setId(Integer.valueOf(s));
                    dataSourceRequestDTOList.add(d);
                }
                else {

                }

            }
            logger.info("start to get ChartDataDTO of:"+objectMapper.writeValueAsString(idList));

            logger.info("start to get ChartDataDTO of:"+objectMapper.writeValueAsString(dataSourceRequestDTOList));
        }
        catch (Exception ex){
            logger.error("get ChartDataDTO error :"+ex.getMessage());
            ex.printStackTrace();
        }

        return dataSourceService.getChartDataDTOList(startTime,endTime,dataSourceRequestDTOList);
    }

    @RequestMapping(value = "/chartDataDTO/{startTime}/{endTime}",method = RequestMethod.GET)
    public Object getDataSourcesByDate(@PathVariable String startTime, @PathVariable String endTime,String param ){
        List<DataSourceRequestDTO> dataSourceRequestDTOList=new ArrayList<>();
        //@RequestBody List<DataSourceRequestDTO> dataSourceRequestDTOList
        try {
            String[] idList=param.split(",");

            for(String s:idList){
                DataSourceRequestDTO d=new DataSourceRequestDTO();
                d.setId(Integer.valueOf(s));
                dataSourceRequestDTOList.add(d);
            }
            logger.info("start to get ChartDataDTO of:"+objectMapper.writeValueAsString(idList));

            logger.info("start to get ChartDataDTO of:"+objectMapper.writeValueAsString(dataSourceRequestDTOList));
        }
        catch (Exception ex){
            logger.error("get ChartDataDTO error :"+ex.getMessage());
            ex.printStackTrace();
        }

//        return "123";
        return dataSourceService.getChartDataDTO(startTime,endTime,dataSourceRequestDTOList);

    }




    @RequestMapping(value = "/dataSources/{id}",method = RequestMethod.DELETE)
    public Object deleteById(@PathVariable int id){
        logger.info("start to delete datasource:"+id);
        return dataSourceService.deleteById(id);
    }


}
