package com.lstznkj.groundstation.model;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.lstznkj.groundstation.utils.DateUtil;
import io.swagger.annotations.ApiModel;
import org.hibernate.annotations.CreationTimestamp;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.core.GrantedAuthority;

import javax.persistence.*;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;

@ApiModel("后端接口权限")
@Entity
public class Authority  implements GrantedAuthority {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "auth_id")
    private Integer id;


    @Column(unique = true,length = 15,nullable = false)
    private String name;


    //后端接口url
    private String apiUrl;
    //接口方法 post get 等等
    private String method;
    @Transient
    private String createTime;
    @Transient
    private String updateTime;


    @CreationTimestamp
    private Timestamp createTimestamp;

    @CreationTimestamp
    private Timestamp updateTimestamp;
    @JsonIgnore
    @ManyToMany(  fetch = FetchType.EAGER,cascade = CascadeType.ALL,mappedBy = "authorities")
    private List<SysRole> sysRoles =new ArrayList<>();

    //权限描述
    private String description;

    //供前端使用
    @Transient
    private boolean editable;




    public Authority(){}

    public Timestamp getCreateTimestamp() {
        return createTimestamp;
    }

    public Timestamp getUpdateTimestamp() {
        return updateTimestamp;
    }

    public void setCreateTimestamp(Timestamp createTimestamp) {
        this.createTimestamp = createTimestamp;
    }

    public void setUpdateTimestamp(Timestamp updateTimestamp) {
        this.updateTimestamp = updateTimestamp;
    }

    public Authority(String authorityStr)
    {
        //需加入authorityStr 检查功能

        Logger logger= LoggerFactory.getLogger( Authority.class);
        String[] strs=authorityStr.split(";");
        for (String s:strs)
        {
            logger.info("s :"+s);
        }
        if(strs.length>=2)
        {
            this.apiUrl=strs[0];
            this.method=strs[1];
        }


    }

    public Integer getId() {
        return id;
    }

    public String getName() {
        return name;
    }


    public void setEditable(boolean editable) {
        this.editable = editable;
    }

    public boolean isEditable() {
        //默认不可编辑
        return false;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public void setName(String name) {
        this.name = name;
    }


    public void setSysRoles(List<SysRole> sysRoles) {
        this.sysRoles = sysRoles;
    }

    public List<SysRole> getSysRoles() {
        return sysRoles;
    }


    public void setApiUrl(String apiUrl) {
        this.apiUrl =  apiUrl;
    }

    public void setMethod(String method) {
        this.method = method;
    }

    public void setDescription(String desc) {
        this.description = desc;
    }

    public String getApiUrl() {
        return apiUrl;
    }

    public String getMethod() {
        return method;
    }

    public String getDescription() {
        return this.description;
    }

    @JsonIgnore
    @Override
    public String getAuthority() {
        return this.apiUrl+";"+this.method;
    }

    public void setCreateTime(String createTime) {
        this.createTime = createTime;
    }

    public void setUpdateTime(String updateTime) {
        this.updateTime = updateTime;
    }

    public String getCreateTime(){
        if(this.createTimestamp==null){
            this.createTime="";
        }else {
            this.createTime= DateUtil.getSingleInstance().formatToMillisecond(this.createTimestamp.getTime());
        }

        return this.createTime;
    }
    public String getUpdateTime(){
        if(this.updateTimestamp==null){
            this.updateTime="";
        }
        else {
            this.updateTime= DateUtil.getSingleInstance().formatToMillisecond(this.updateTimestamp.getTime());
        }

        return this.updateTime;
    }

}
