package com.lstznkj.groundstation.model;

import com.lstznkj.groundstation.utils.DateUtil;
import io.swagger.annotations.ApiModel;
import org.hibernate.annotations.CreationTimestamp;
import org.hibernate.annotations.UpdateTimestamp;

import javax.persistence.*;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;



@ApiModel("用户")
@Entity
public class User {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "user_id")
    private Integer id;
    @Column(nullable = false)
    private String name;
    @Column(unique = true,length = 15,nullable = false)
    private String username;


    @Column(nullable = false)
    private String password;

    private boolean locked;

    @Transient
    private String createTime;
    @Transient
    private String updateTime;
    @CreationTimestamp
    private Timestamp createTimestamp;
    @UpdateTimestamp
    private Timestamp updateTimestamp;
    //备注
    @Column(length = 255)
    private String remark;

    @ManyToMany(fetch = FetchType.EAGER)
    @JoinTable(name ="user_sys_roles", joinColumns = {@JoinColumn(name = "user_id",referencedColumnName = "user_id")},
            inverseJoinColumns = {@JoinColumn(name = "role_id",referencedColumnName = "role_id")})
    private List<SysRole> sysRoles;


    ///方便前端获取roles
    @Transient
    private List<String> roles=new ArrayList<>();

    public String getRemark() {
        return remark;
    }

    public void setRemark(String remark) {
        this.remark = remark;
    }



    public void setSysRoles(List<SysRole> sysRoles) {
        this.sysRoles = sysRoles;
    }

//    @JsonIgnore
    public List<SysRole> getSysRoles() {
        return sysRoles;
    }

    public List<String> getRoles() {
        if(this.roles.size()==0)
        {
            if(sysRoles!=null)
            {
                for(SysRole role : sysRoles)
                {
                    this.roles.add(role.getName());
                }
            }

        }

        return this.roles;
    }

    public Integer getId() {
        return id;
    }

    public String getName() {
        return name;
    }

    public String getUsername() {
        return username;
    }

    public String getPassword() {
        return password;
    }

    public boolean isLocked() {
        return locked;
    }

    public Timestamp getCreateTimestamp() {
        return createTimestamp;
    }

    public Timestamp getUpdateTimestamp() {
        return updateTimestamp;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public void setName(String name) {
        this.name = name;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public void setLocked(boolean locked) {
        this.locked = locked;
    }

    public void setCreateTimestamp(Timestamp createTimestamp) {
        this.createTimestamp = createTimestamp;
    }

    public void setUpdateTimestamp(Timestamp updateTimestamp) {
        this.updateTimestamp = updateTimestamp;
    }

    public String getCreateTime(){
        if(this.createTimestamp==null){
            this.createTime="";
        }else {
            this.createTime= DateUtil.getSingleInstance().formatToMillisecond(this.createTimestamp.getTime());
        }

        return this.createTime;
    }
    public String getUpdateTime(){
        if(this.updateTimestamp==null){
            this.updateTime="";
        }
        else {
            this.updateTime= DateUtil.getSingleInstance().formatToMillisecond(this.updateTimestamp.getTime());
        }

        return this.updateTime;
    }

    public void setCreateTime(String createTime) {
        this.createTime = createTime;
    }

    public void setUpdateTime(String updateTime) {
        this.updateTime = updateTime;
    }
}
