package com.lstznkj.groundstation.model.crane;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.lstznkj.groundstation.utils.DateUtil;
import org.hibernate.annotations.CreationTimestamp;
import org.hibernate.annotations.UpdateTimestamp;

import javax.persistence.*;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;

@Entity
public class DataSource {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private int id;
    private String name;

    //是否启用
    private Boolean isEnable;

    //采集地址
    private String address;




    @ManyToOne
    private DataType dataType;

    //mqtt发布主题名称
    private String publishTopic;
    @CreationTimestamp
    private Timestamp createTimestamp;
    @UpdateTimestamp
    private Timestamp updateTimestamp;
    @Transient
    private String createTime;
    @Transient
    private int dataTypeId;
    @Transient
    private String updateTime;
    @Transient
    private List<String> valueList=new ArrayList<>();

    @Transient
    private List<String> dateList=new ArrayList<>();
    //报警阈值
    private float threshold;
    private float maxValue;
    private float minValue;


    //稀疏差异值(大于等于稀疏差异值 才存储到数据库),-1时，不进行稀疏存储
    private double differSparseValue;


    private int sparseCount;

    //累计 稀疏值，(超过累计稀疏值后 存储到数据库)

    private double totalSparseValue;

    @Transient
    private int tmpSparseCount;


    @Transient
    private BaseData lastBaseData;

    @Transient
    private double differTotalValue;

    @JsonIgnore
    @ManyToOne
    private Device device;
    @Transient
    private String deviceInfo;
    @Transient
    private String dataTypeInfo;

    @Transient
    private int deviceId;

    @Transient
    private String deviceName;

    private String remark;

    @JsonIgnore
    @OneToMany(mappedBy = "dataSource",cascade = CascadeType.ALL)
    private List<BaseData> baseDataList;



    public int getDeviceId() {

        if(this.device!=null){
            return this.device.getId();
        }
        else {
            return deviceId;
        }

    }

    public String getDeviceName() {
        if(this.device!=null){
            this.deviceName=this.device.getName();
        }
        return deviceName;
    }

    public void setDeviceName(String deviceName) {
        this.deviceName = deviceName;
    }

    public void setDeviceId(int deviceId) {
        this.deviceId = deviceId;
    }

    public void setDeviceInfo(String deviceInfo) {
        this.deviceInfo = deviceInfo;
    }

    public void setDataTypeInfo(String dataTypeInfo) {
        this.dataTypeInfo = dataTypeInfo;
    }

    public List<String> getDateList() {
        return dateList;
    }

    public void setDateList(List<String> dateList) {
        this.dateList = dateList;
    }

    public void setValueList(List<String> valueList) {
        this.valueList = valueList;
    }

    public List<String> getValueList() {
        return valueList;
    }

    public void setMaxValue(float maxValue) {
        this.maxValue = maxValue;
    }

    public void setMinValue(float minValue) {
        this.minValue = minValue;
    }

    public BaseData getLastBaseData() {
        return lastBaseData;
    }

    public void setLastBaseData(BaseData lastBaseData) {
        this.lastBaseData = lastBaseData;
    }

    public String getDeviceInfo() {
        if(device!=null){
            deviceInfo=device.getFullName();
        }
        return deviceInfo;
    }

    public String getDataTypeInfo() {
        if(dataType!=null){
            dataTypeInfo=dataType.getName()+","+ dataType.getRemark();
        }
        return dataTypeInfo;
    }


    public float getMaxValue() {
        return maxValue;
    }

    public float getMinValue() {
        return minValue;
    }

    public String getRemark() {
        return remark;
    }

    public void setRemark(String remark) {
        this.remark = remark;
    }

    public void setPublishTopic(String publishTopic) {
        this.publishTopic = publishTopic;
    }

    public String getPublishTopic() {
        return publishTopic;
    }

    public Boolean getEnable() {
        return isEnable;
    }

    public void setEnable(Boolean enable) {
        isEnable = enable;
    }

    public int getDataTypeId() {
        //return dataTypeId;
        if(this.dataType!=null){
            return this.dataType.getId();
        }
        else{
            return  this.dataTypeId;
        }

    }

    public void setDataTypeId(int dataTypeId) {
        this.dataTypeId = dataTypeId;
    }





    public List<BaseData> getBaseDataList() {
        return baseDataList;
    }

    
    public void setCreateTimestamp(Timestamp createTimestamp) {
        this.createTimestamp = createTimestamp;
    }

    public void setUpdateTimestamp(Timestamp updateTimestamp) {
        this.updateTimestamp = updateTimestamp;
    }

    public Timestamp getCreateTimestamp() {
        return createTimestamp;
    }

    public Timestamp getUpdateTimestamp() {
        return updateTimestamp;
    }



    public void setBaseDataList(List<BaseData> baseDataList) {
        this.baseDataList = baseDataList;
    }

    public Device getDevice() {
        return device;
    }

    public void setDevice(Device device) {
        this.device = device;
    }

    //报警阈值


    public DataType getDataType() {
        return dataType;
    }

    public void setDataType(DataType dataType) {
        this.dataType = dataType;
    }


    public int getTmpSparseCount() {
        return tmpSparseCount;
    }

    public void setTmpSparseCount(int tmpSparseCount) {
        this.tmpSparseCount = tmpSparseCount;
    }

    public String getAddress() {
        return address;
    }

    public void setAddress(String address) {
        this.address = address;
    }

    public void setId(int id) {
        this.id = id;
    }

    public void setName(String name) {
        this.name = name;
    }

    public void setThreshold(float threshold) {
        this.threshold = threshold;
    }

    public void setCreateTime(String createTime) {
        this.createTime = createTime;
    }

    public int getId() {
        return id;
    }

    public String getName() {
        return name;
    }

    public float getThreshold() {
        return threshold;
    }

    public void setUpdateTime(String updateTime) {
        this.updateTime = updateTime;
    }

    public double getDifferSparseValue() {
        return differSparseValue;
    }

    public void setDifferSparseValue(double differSparseValue) {
        this.differSparseValue = differSparseValue;
    }

    public double getDifferTotalValue() {
        return differTotalValue;
    }

    public void setDifferTotalValue(double differTotalValue) {
        this.differTotalValue = differTotalValue;
    }

    public String getCreateTime(){
        if(this.createTimestamp==null){
            this.createTime="";
        }else {
            this.createTime= DateUtil.getSingleInstance().formatToSecond(this.createTimestamp.getTime());
        }

        return this.createTime;
    }

    public int getSparseCount() {
        return sparseCount;
    }

    public void setSparseCount(int sparseCount) {
        this.sparseCount = sparseCount;
    }

    public String getUpdateTime(){
        if(this.updateTimestamp==null){
            this.updateTime="";
        }
        else {
            this.updateTime= DateUtil.getSingleInstance().formatToSecond(this.updateTimestamp.getTime());
        }

        return this.updateTime;
    }

    public double getTotalSparseValue() {
        return totalSparseValue;
    }

    public void setTotalSparseValue(double totalSparseValue) {
        this.totalSparseValue = totalSparseValue;
    }
}
