package com.lstznkj.groundstation.mqtt;

import com.lstznkj.groundstation.GlobalData;
import org.eclipse.paho.client.mqttv3.MqttClient;
import org.eclipse.paho.client.mqttv3.MqttConnectOptions;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;

public class MqttFactory {
    private static MqttClient mqttClient;
    public static void setInstance(MqttClient client){
        mqttClient=client;
    }
    public static MqttClient getInstance(){
        return mqttClient;
    }
    public static MqttProvider createMqttProvider(String username,String password,String url,String id){
        MqttProvider mqttProvider=new MqttProvider();


        try{
            //创建MQTT客户端对象
            MqttClient  client = new MqttClient(url,id,new MemoryPersistence());
            //连接设置
            MqttConnectOptions options = new MqttConnectOptions();
            //是否清空session，设置false表示服务器会保留客户端的连接记录（订阅主题，qos）,客户端重连之后能获取到服务器在客户端断开连接期间推送的消息
            //设置为true表示每次连接服务器都是以新的身份
            options.setCleanSession(true);

            //设置连接用户名
            options.setUserName(username);
            //设置连接密码
            options.setPassword(password.toCharArray());
            //设置超时时间，单位为秒
            options.setConnectionTimeout(100);
            //设置心跳时间 单位为秒，表示服务器每隔 1.5*20秒的时间向客户端发送心跳判断客户端是否在线
            options.setKeepAliveInterval(20);
            //设置遗嘱消息的话题，若客户端和服务器之间的连接意外断开，服务器将发布客户端的遗嘱信息
            options.setWill("willTopic",(id + "与服务器断开连接").getBytes(),0,false);
            //设置回调
           // client.setCallback(new MqttProviderCallBack());
            mqttProvider.setClientId(id);
            MqttProviderCallBack callBack=new MqttProviderCallBack();
            callBack.setClientId(id);
            mqttProvider.setClient(client);
            mqttProvider.setCallBack(callBack);
            mqttProvider.connect(options);

           // client.connect(options);


            mqttProvider.setConnected(true);

        } catch(MqttException e){
            mqttProvider.setConnected(false);
            e.printStackTrace();
        }

        return mqttProvider;
    }

    public static MqttClient createInstance(String username,String password,String url,String id){
        MqttClient client=null;
        try {
            client =new MqttClient(url,id,new MemoryPersistence() );
            MqttConnectOptions options = new MqttConnectOptions();
            //是否清空session，设置false表示服务器会保留客户端的连接记录（订阅主题，qos）,客户端重连之后能获取到服务器在客户端断开连接期间推送的消息
            //设置为true表示每次连接服务器都是以新的身份
//            options.setCleanSession(true);
            options.setCleanSession(false);
            //设置连接用户名
            options.setUserName(username);
            //设置连接密码
            options.setPassword(password.toCharArray());
            //设置超时时间，单位为秒
            options.setConnectionTimeout(100);
            //设置心跳时间 单位为秒，表示服务器每隔 1.5*20秒的时间向客户端发送心跳判断客户端是否在线
            options.setKeepAliveInterval(20);

            options.setAutomaticReconnect(true);
            //设置遗嘱消息的话题，若客户端和服务器之间的连接意外断开，服务器将发布客户端的遗嘱信息
            options.setWill("willTopic",(id + "与服务器断开连接").getBytes(),0,false);
            //设置回调
            MqttProviderCallBack callBack=new MqttProviderCallBack();
            callBack.setOptions(options);
            callBack.setClientId(id);
            client.setCallback(callBack);
            client.connect(options);
            callBack.setClient(client);
        }
        catch (Exception ex){
            ex.printStackTrace();
        }
        return client;

    }
}
