package com.lstznkj.groundstation.security;

import com.lstznkj.groundstation.security.jwt.JWTAuthenticationFilter;
import com.lstznkj.groundstation.security.jwt.JWTLoginFilter;
import com.lstznkj.groundstation.security.jwt.JWTUtils;
import com.lstznkj.groundstation.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.HttpMethod;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;

@Configuration
@EnableWebSecurity

public class WebSecurityConfig extends WebSecurityConfigurerAdapter {
    @Autowired
    UserDetailsServiceImpl userDetailsService;

    @Autowired
    UrlAccessDecisionManager accessDecisionManager;
    @Autowired
    UserService userService;


    @Autowired
    //加密方式
    private BCryptPasswordEncoder bCryptPasswordEncoder;


    @Override
    protected void configure(AuthenticationManagerBuilder auth) throws Exception {
        //注入userDetailService

        auth.userDetailsService(userDetailsService)
                .passwordEncoder(bCryptPasswordEncoder);//加密方式
    }



    @Override
    protected void configure(HttpSecurity http) throws Exception {

        http.cors().and().csrf().disable().authorizeRequests()
                .antMatchers(HttpMethod.POST, "/login").permitAll()
                .antMatchers(HttpMethod.POST, "/test/**").permitAll()
                .antMatchers(HttpMethod.POST, "/devices/*/dataSources").permitAll()
                .antMatchers(HttpMethod.POST, "/logout").permitAll()
                .antMatchers(HttpMethod.POST,"/campus-recruitments").permitAll()
                .antMatchers(HttpMethod.GET,"/swagger-ui.html","/swagger*/**","/v2/**","/webjars/**").permitAll()
                .antMatchers(HttpMethod.GET,"/users/secret/**").permitAll()
                .antMatchers(HttpMethod.PUT,"/users/secret/**").permitAll()
                //.antMatchers(HttpMethod.GET,"/upload-dir/*").permitAll()
                .anyRequest().authenticated()
                .accessDecisionManager(accessDecisionManager)
                .and()
                .addFilter(new JWTAuthenticationFilter(authenticationManager()))
                // .formLogin().loginProcessingUrl("/login")
                //.successHandler(loginSuccessHandler);
                .addFilter(new JWTLoginFilter(authenticationManager()));
    }

/*    @Bean
    public BCryptPasswordEncoder bCryptPasswordEncoder() {
        return new BCryptPasswordEncoder();
    }*/


}