package com.lstznkj.groundstation.service;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.lstznkj.groundstation.GlobalData;
import com.lstznkj.groundstation.model.Result;
import com.lstznkj.groundstation.model.crane.BaseData;
import com.lstznkj.groundstation.model.crane.DataSource;
import com.lstznkj.groundstation.model.crane.Device;
import com.lstznkj.groundstation.model.crane.ThreadMonitorInfo;
import com.lstznkj.groundstation.mqtt.MqttFactory;
import com.lstznkj.groundstation.mqtt.MqttManager;
import com.lstznkj.groundstation.mqtt.MqttProvider;
import com.lstznkj.groundstation.service.crane.BaseDataService;
import com.lstznkj.groundstation.service.crane.DataSourceService;
import com.lstznkj.groundstation.service.crane.DeviceService;
import com.lstznkj.groundstation.utils.SpringContextUtil;
import org.eclipse.paho.client.mqttv3.IMqttMessageListener;
import org.eclipse.paho.client.mqttv3.MqttClient;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 数据整合服务，订阅所有天车mqtt 主题并存储
 */

@Service
public class DataIntegrateService {
    private Logger logger= LoggerFactory.getLogger(getClass());
    @Autowired
    private DataSourceService dataSourceService;
    private boolean isRunningCollectThreads;
//    @Autowired
//    private BaseDataService baseDataService;
    @Autowired
    private DeviceService deviceService;
    @Autowired
    private ObjectMapper objectMapper;

    private ExecutorService suscribeThreadPool;

    private ExecutorService dataSaveThreadPool;



    @PostConstruct
    public void init(){
        dataSaveThreadPool= Executors.newCachedThreadPool();
    }

    /**
     * 启动 数据采集线程，每台天车 启动一个采集线程
     *
     */
    public void startDataCollectThreads(){
        int threadCount=deviceService.getRootDevices().getData().size();
        logger.info("thread pool size :"+threadCount);
        if(threadCount==0){
            logger.info("root devices is 0,stop to collect data");
            return;
        }
        this.suscribeThreadPool = Executors.newFixedThreadPool(threadCount);

        Result<List<Device>> rootDeviceList=deviceService.getRootDevices();

        //start to create threadMonitor info


        if(rootDeviceList.isSuccess()){
            for(Device root:rootDeviceList.getData()){
                //start to create threadMonitor info
                ThreadMonitorInfo threadMonitorInfo=new ThreadMonitorInfo();
                threadMonitorInfo.setRunable(true);
                threadMonitorInfo.setRunTime(0);
                GlobalData.threadMonitorInfoList.add(threadMonitorInfo);
                //end create threadMonitor info
                Thread thread=new Thread(new Runnable() {
                    @Override
                    public void run() {


                        logger.info("device id "+root.getId());
                        String mqttURL="tcp://"+root.getExtra()+":1883";
                        logger.info(root.getName()+":"+mqttURL+" start to collect data");
                        MqttProvider provider=MqttFactory.createMqttProvider("admin","123456",mqttURL,root.getExtra());


                        MqttClient client=provider.getClient();
                       // MqttClient client= MqttFactory.createInstance("admin","123456",mqttURL,root.getExtra());


                        logger.info(root.getName()+" mqtt client connected");
                        Result<List<DataSource>> dataSourceList=null;
                        try {
                           dataSourceList=dataSourceService.getAllDataSources(root.getId());

                           logger.info("dataSourceList:"+objectMapper.writeValueAsString(dataSourceList));

                            provider.setDataSourceList(dataSourceList.getData());

                            //inster to MqttManager list
                            MqttManager.addMqttProvider(provider);

                            provider.subscribeDataSource();


                   /*         for(DataSource dataSource:dataSourceList.getData()){

                                client.subscribe(dataSource.getPublishTopic(), new IMqttMessageListener() {
                                    @Override
                                    public void messageArrived(String s, MqttMessage mqttMessage) throws Exception {


                                        Thread tmpThread=new Thread(new Runnable() {

                                            private BaseDataService baseDataService= SpringContextUtil.getBean(BaseDataService.class);
                                            @Override
                                            public void run() {
                                                try {
                                                    BaseData baseData=new BaseData();
                                                    JsonNode mqttNode=objectMapper.readTree(mqttMessage.toString());
                                                    baseData.setName(mqttNode.get("name").asText());
                                                    baseData.setValue(mqttNode.get("value").asDouble());
                                                    baseData.setOriginValue(mqttNode.get("originValue").asInt());
                                                    baseData.setCollectTime(mqttNode.get("createTime").asLong());
                                                    baseData=baseDataService.add(baseData).getData();
                                                    baseData.setDataSource(dataSource);
                                                    baseData=baseDataService.add(baseData).getData();
                                                    logger.info("baseData:"+objectMapper.writeValueAsString(baseData));
                                                }
                                                catch (Exception ex){
                                                    logger.error("base data save to database error :"+ex.getMessage());
                                                    ex.printStackTrace();
                                                }

                                            }
                                        });
                                        dataSaveThreadPool.submit(tmpThread);



//                                           BaseData baseData=objectMapper.readValue(mqttMessage.toString(),BaseData.class);
//                                           logger.info("subscribe: "+s+"; "+objectMapper.writeValueAsString(baseData));

                                        logger.info("topic:"+s+": "+mqttMessage.toString());
                                    }
                                });
                            }
*/
                        }
                        catch (Exception ex){
                            logger.error(ex.getMessage());
                        }



                  /*      while (threadMonitorInfo.isRunable()){
                            threadMonitorInfo.setRunTime(threadMonitorInfo.getRunTime()+1);
                           try {
                               if(dataSourceList==null){
                               }
                               for(DataSource dataSource:dataSourceList.getData()){

                                   client.subscribe(dataSource.getPublishTopic(), new IMqttMessageListener() {
                                       @Override
                                       public void messageArrived(String s, MqttMessage mqttMessage) throws Exception {


                                           Thread tmpThread=new Thread(new Runnable() {

                                               private BaseDataService baseDataService= SpringContextUtil.getBean(BaseDataService.class);
                                               @Override
                                               public void run() {
                                                   try {
                                                       BaseData baseData=new BaseData();
                                                       JsonNode mqttNode=objectMapper.readTree(mqttMessage.toString());
                                                       baseData.setName(mqttNode.get("name").asText());
                                                       baseData.setValue(mqttNode.get("value").asDouble());
                                                       baseData.setOriginValue(mqttNode.get("originValue").asInt());
                                                       baseData.setCollectTime(mqttNode.get("createTime").asLong());
                                                       baseData=baseDataService.add(baseData).getData();
                                                       baseData.setDataSource(dataSource);
                                                       baseData=baseDataService.add(baseData).getData();
                                                       logger.info("baseData:"+objectMapper.writeValueAsString(baseData));
                                                   }
                                                   catch (Exception ex){
                                                       logger.error("base data save to database error :"+ex.getMessage());
                                                       ex.printStackTrace();
                                                   }

                                               }
                                           });
                                           dataSaveThreadPool.submit(tmpThread);



//                                           BaseData baseData=objectMapper.readValue(mqttMessage.toString(),BaseData.class);
//                                           logger.info("subscribe: "+s+"; "+objectMapper.writeValueAsString(baseData));

                                           logger.info("topic:"+s+": "+mqttMessage.toString());
                                       }
                                   });
                               }

                           }
                           catch (Exception ex){
                               logger.error(root.getExtra()+" mqttServer  connected!!! error!"+ex.getMessage());
//                               logger.error("start to reconnect to "+root.getExtra()+" mqttserver!!!");
//                               client= MqttFactory.createInstance("admin","123456",mqttURL,String.valueOf(root.getId()));
//                               logger.info(root.getName()+" mqtt client connected");

                               threadMonitorInfo.setRunable(false);
                               ex.printStackTrace();
                           }


                        }*/
                        threadMonitorInfo.setRunable(false);
                    }
                });
                thread.setName(root.getId()+root.getName());

                suscribeThreadPool.submit(thread);
                logger.info("create data collect thread:"+thread.getName());
            }
        }
    }

    public void stopDataCollectThreads(){
        try {
            suscribeThreadPool.shutdown();
        }
        catch (Exception ex){
            ex.printStackTrace();
        }
    }



}
