package com.lstznkj.groundstation.service;

import com.lstznkj.groundstation.model.Authority;
import com.lstznkj.groundstation.model.Result;
import com.lstznkj.groundstation.model.SysRole;
import com.lstznkj.groundstation.model.User;
import com.lstznkj.groundstation.repo.AuthorityRepository;
import com.lstznkj.groundstation.repo.SysRoleRepository;
import com.lstznkj.groundstation.repo.UserRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;


@Service
public class SysRoleService {
    @Autowired
    private SysRoleRepository sysRoleRepository;
    @Autowired
    AuthorityRepository authorityRepository;
    @Autowired
    UserRepository userRepository;

    private Logger logger= LoggerFactory.getLogger(SysRoleService.class);

    /**
     * 查找所有角色
     * @return
     */
    public Result<Iterable<SysRole>> findAll()
    {
        Result<Iterable<SysRole>> result=new Result<>();
        try {
            result.setData(sysRoleRepository.findAll());
            result.setSuccess(true);
        }
        catch (Exception ex)
        {
            result.setSuccess(false);
            ex.printStackTrace();
        }
        return result;
    }
    public SysRole findSysRoleByName(String name)
    {
        return sysRoleRepository.findSysRoleByName(name);
    }


    public SysRole getAdminSysRole()
    {
        return sysRoleRepository.findSysRoleByName("ROLE_ADMIN");
    }
    public SysRole getUltimateSysRole()
    {
        return sysRoleRepository.findSysRoleByName("ROLE_ULTIMATE");
    }
    public SysRole getBudgetSysRole()
    {
        return sysRoleRepository.findSysRoleByName("ROLE_BUDGET");
    }



    public Result<SysRole> deleteById(int id)
    {
        Result<SysRole> result=new Result<>();
        try {
            if(sysRoleRepository.existsById(id)){
                SysRole role=sysRoleRepository.findSysRoleById(id);
                for(Authority auth : role.getAuthorities())
                {
                    auth.getSysRoles().remove(role);
                    authorityRepository.save(auth);
                }
                for(User user:role.getUsers()){
                    user.getSysRoles().remove(role);
                    userRepository.save(user);
                }
                role.setUsers(null);
                role.setAuthorities(null);
                sysRoleRepository.save(role);
                sysRoleRepository.deleteById(id);
                result.setSuccess(true);
            }
            else {
                result.setSuccess(false);
                result.setInfo("不存在此角色");
            }
        }catch (Exception ex){
            ex.printStackTrace();
            result.setSuccess(false);
            result.setInfo("未知错误!!!");
        }
        return result;
    }

    public Result<SysRole> save(SysRole sysRole)
    {
        Result<SysRole> result=new Result<>();
        try {
            if(sysRoleRepository.existsSysRoleByName(sysRole.getName()))
            {
                result.setSuccess(false);
                result.setInfo("已经存在此名称的角色!!!");
                return result;
            }
            if(sysRoleRepository.save(sysRole)!=null) {
                result.setSuccess(true);
                return result;
            }
            else {
                result.setSuccess(false);
                result.setInfo("未知错误!!!");
                return result;
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
            result.setSuccess(false);
            result.setInfo("未知错误!!!");
            return result;
        }
    }

    public Result<SysRole> update(SysRole sysRole)
    {
        Result<SysRole> result=new Result<>();

        try {
            if(sysRoleRepository.existsById(sysRole.getId()))
            {
                SysRole oldRole=sysRoleRepository.findSysRoleById(sysRole.getId());
                oldRole.setDescription(sysRole.getDescription());
                oldRole.setAuthorities(sysRole.getAuthorities());
                oldRole.setName(sysRole.getName());

                oldRole=sysRoleRepository.save(oldRole);
                result.setData(oldRole);
                result.setSuccess(true);
            }else {
                result.setSuccess(false);
                result.setInfo("不存在此角色");
            }

        }catch (Exception ex)
        {
            ex.printStackTrace();
            result.setSuccess(false);
            result.setInfo("未知错误!!!");
        }
        return result;
    }
}
