package com.lstznkj.groundstation.service.crane;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.lstznkj.groundstation.DTO.ui.CraneDTO;
import com.lstznkj.groundstation.DTO.ui.CraneSummaryDTO;
import com.lstznkj.groundstation.DTO.ui.TechParamsDTO;
import com.lstznkj.groundstation.model.Result;
import com.lstznkj.groundstation.model.crane.Device;
import com.lstznkj.groundstation.model.crane.LifeModel;
import com.lstznkj.groundstation.model.crane.extra.DeviceSummary;
import com.lstznkj.groundstation.model.crane.extra.Equipment;
import com.lstznkj.groundstation.model.crane.extra.TechParameter;
import com.lstznkj.groundstation.repo.crane.DeviceRepository;
import com.lstznkj.groundstation.repo.crane.extra.DeviceSummaryRepository;
import com.lstznkj.groundstation.repo.crane.extra.EquipmentRepository;
import com.lstznkj.groundstation.repo.crane.extra.LifeModelRepository;
import com.lstznkj.groundstation.repo.crane.extra.TechParameterRepository;
import com.lstznkj.groundstation.utils.DateUtil;
import com.lstznkj.groundstation.utils.ObjectUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;

@Service
public class CraneService {
    private Logger logger= LoggerFactory.getLogger(getClass());
    @Autowired
    private DateUtil dateUtil;
    @Autowired
    private DeviceRepository deviceRepository;
    @Autowired
    private DeviceSummaryRepository deviceSummaryRepository;
    @Autowired
    private TechParameterRepository techParameterRepository;
    @Autowired
    private EquipmentRepository equipmentRepository;
    @Autowired
    private LifeModelRepository lifeModelRepository;
    @Autowired
    private ObjectMapper objectMapper;
    @Autowired
    private ObjectUtil objectUtil;
    public Result<Device> addCrane(CraneDTO craneDTO){
        Result<Device> result=new Result<>();
        try {

            Device crane=transform(craneDTO);
            DeviceSummary summary=transform(craneDTO.getCraneSummaryDTO());
            TechParameter techParameter=transform(craneDTO.getTechParamsDTO());

            crane=deviceRepository.save(crane);

            summary=deviceSummaryRepository.save(summary);
            summary.setCrane(crane);
            summary=deviceSummaryRepository.save(summary);

            techParameter=techParameterRepository.save(techParameter);
            techParameter.setCrane(crane);
            techParameter=techParameterRepository.save(techParameter);


            result.setSuccess(true);
        }
        catch (Exception ex){
            logger.error("add Crane error:"+ex.getMessage());
            result.setSuccess(false);
        }
        return result;
    }
    public Result<Device>  updateCrane(int id, CraneDTO craneDTO){
        Result<Device> result=new Result<>();
        try {

            Device newCrane=transform(craneDTO);
            DeviceSummary newSummary=transform(craneDTO.getCraneSummaryDTO());
            TechParameter newTechParameter=transform(craneDTO.getTechParamsDTO());

            Device originCrane=deviceRepository.findDeviceById(id);
            Timestamp createTimestamp=originCrane.getCreateTimestamp();
            TechParameter originTechParameter=techParameterRepository.findByCrane_Id(originCrane.getId());
            DeviceSummary originSummary=deviceSummaryRepository.findByCrane_Id(originCrane.getId());

            originCrane=objectUtil.deepCopy(originCrane,newCrane);
            originCrane.setCreateTimestamp(createTimestamp);
            originSummary=objectUtil.deepCopy(originSummary,newSummary);
            originTechParameter=objectUtil.deepCopy(originTechParameter,newTechParameter);

            originCrane=deviceRepository.save(originCrane);


            originSummary.setCrane(originCrane);
            originSummary.setName(originCrane.getName());
            originSummary=deviceSummaryRepository.save(originSummary);

            originTechParameter.setCrane(originCrane);
            originTechParameter=techParameterRepository.save(originTechParameter);



            result.setSuccess(true);
        }
        catch (Exception ex){
            logger.error("update Crane "+craneDTO.getCraneSummaryDTO().getName()+"  error:"+ex.getMessage());
            ex.printStackTrace();
            result.setSuccess(false);
        }
        return result;
    }


    /**
     * 递归删除设备及其所有子设备
     * @param device
     */
    private void deleteDeviceAndChildren(Device device){
        if(device.getChildren()!=null){
            for(Device child: device.getChildren()){
                deleteDeviceAndChildren(child);
            }
        }

        DeviceSummary summary=deviceSummaryRepository.findByCrane_Id(device.getId());
        TechParameter techParameter=techParameterRepository.findByCrane_Id(device.getId());
        Equipment equipment=equipmentRepository.findByDevice_Id(device.getId());
        LifeModel lifeModel=lifeModelRepository.findByDevice_Id(device.getId());

        if(summary!=null){
            deviceSummaryRepository.deleteById(summary.getId());
        }
        if(techParameter!=null){
            techParameterRepository.deleteById(techParameter.getId());
        }


//        equipment.setDevice(null);
//        equipment.setOldDeviceId(device.getId());
//        lifeModel.setDevice(null);
//        lifeModel.setOldDeviceId(device.getId());
//        equipment=equipmentRepository.save(equipment);
//        lifeModel=lifeModelRepository.save(lifeModel);

        if(lifeModel!=null){
            lifeModelRepository.deleteById(lifeModel.getId());
        }
        if(equipment!=null){
            equipmentRepository.deleteById(equipment.getId());
        }





        deviceRepository.delete(device);

    }
    /**
     * 删除指定id的设备
     * @return
     */


    public Result<Device> deleteCraneById(int id){
        Result<Device> result=new Result<>();
        try {
            if(deviceRepository.existsById(id)){
                Device device=deviceRepository.findDeviceById(id);
                deleteDeviceAndChildren(device);
                result.setSuccess(true);
            }
            else {
                result.setSuccess(false);
                result.setInfo("不存在此设备!");
            }
        }
        catch (Exception ex){
            ex.printStackTrace();
            result.setSuccess(false);
            result.setInfo("未知错误!");
        }
        return result;
    }


    public Result<List<CraneDTO>> getAllCranes()
    {
        Result<List<CraneDTO>> result=new Result<>();
        List<CraneDTO> craneDTOList=new ArrayList<>();
        List<Device> craneList=null;
        try {
            logger.info("start to getAllCranes");
             craneList= deviceRepository.findByLevelIs(0);
             if(craneList!=null){
                 for(Device c:craneList){
                     DeviceSummary summary=deviceSummaryRepository.findByCrane_Id(c.getId());
                     TechParameter techParameter=techParameterRepository.findByCrane_Id(c.getId());
                     CraneSummaryDTO craneSummaryDTO=new CraneSummaryDTO(summary);
                     TechParamsDTO techParamsDTO=new TechParamsDTO(techParameter);
                     CraneDTO craneDTO=new CraneDTO(c);
                     craneDTO.setTechParamsDTO(techParamsDTO);
                     craneDTO.setCraneSummaryDTO(craneSummaryDTO);
                     logger.debug("getAllCraneDTOs:"+objectMapper.writeValueAsString(craneDTO));
                     craneDTOList.add(craneDTO);
                 }
                 result.setData(craneDTOList);
                 result.setSuccess(true);
             }
             else {
                 logger.error("crane List is null");
             }
        }
        catch (Exception ex){
            logger.error("get crane List is eror:"+ex.getMessage());
            result.setSuccess(false);
            ex.printStackTrace();
        }

        return result;

    }
    public Device transform(CraneDTO craneDTO){
        Device crane=new Device();
        crane.setId(craneDTO.getId());
        crane.setName(craneDTO.getName());
        crane.setRemark(crane.getRemark());
        crane.setState(craneDTO.getState());
        crane.setExtra(craneDTO.getExtra());


        return crane;
    }


    public DeviceSummary transform(CraneSummaryDTO summaryDTO){
        DeviceSummary summary=new DeviceSummary();
        summary.setId(summaryDTO.getId());
        summary.setName(summaryDTO.getName());
        summary.setSpecification(summaryDTO.getSpecification());
        summary.setProduceCompany(summaryDTO.getProduceCompany());
        summary.setProduceNumber(summaryDTO.getProduceNumber());
        summary.setProduceTime(dateUtil.parseYYYY_MM_DD(summaryDTO.getProduceTime()).getTime());
        summary.setInstallTime(dateUtil.parseYYYY_MM_DD(summaryDTO.getInstallTime()).getTime());
        summary.setStartTime(dateUtil.parseYYYY_MM_DD(summaryDTO.getStartTime()).getTime());
        summary.setOriginPrice(summaryDTO.getOriginPrice());
        summary.setOriginPriceUnit(summaryDTO.getOriginPriceUnit());
        summary.setAgeLimit(summaryDTO.getAgeLimit());
        summary.setProfile(summaryDTO.getProfile());
        summary.setImageNumber(summaryDTO.getImageNumber());
        summary.setUsedUnit(summaryDTO.getUsedUnit());
        summary.setIntroduction(summaryDTO.getIntroduction());

        return  summary;
    }
    public TechParameter transform(TechParamsDTO techParamsDTO){

        TechParameter techParameter=new TechParameter();
        techParameter.setId(techParamsDTO.getId());
        techParameter.setLength(techParamsDTO.getLength());
        techParameter.setLengthUnit(techParamsDTO.getLengthUnit());
        techParameter.setWidth(techParamsDTO.getWidth());
        techParameter.setWidthUnit(techParamsDTO.getWidthUnit());
        techParameter.setHighestValueToRail(techParamsDTO.getHighestValueToRail());
        techParameter.setHighestValueToRailUnit(techParamsDTO.getHighestValueToRailUnit());

        techParameter.setRailSurfaceHeight(techParamsDTO.getRailSurfaceHeight());
        techParameter.setRailSurfaceHeightUnit(techParamsDTO.getRailSurfaceHeightUnit());

        techParameter.setBigCarBufferDistance(techParamsDTO.getBigCarBufferDistance());
        techParameter.setBigCarBufferDistanceUnit(techParamsDTO.getBigCarBufferDistanceUnit());

        techParameter.setSmallCarBufferDistance(techParamsDTO.getSmallCarBufferDistance());
        techParameter.setSmallCarBufferDistanceUnit(techParamsDTO.getSmallCarBufferDistanceUnit());

        techParameter.setPowerGenerationgTotalCapacity(techParamsDTO.getPowerGenerationgTotalCapacity());
        techParameter.setPowerGenerationgTotalCapacityUnit(techParamsDTO.getPowerGenerationgTotalCapacityUnit());

        techParameter.setMainHoistHeight(techParamsDTO.getMainHoistHeight());
        techParameter.setMainHoistHeightUnit(techParamsDTO.getMainHoistHeightUnit());

        techParameter.setDeputyHoistHeight(techParamsDTO.getDeputyHoistHeight());
        techParameter.setDeputyHoistHeightUnit(techParamsDTO.getDeputyHoistHeightUnit());

        techParameter.setBigCarLength(techParamsDTO.getBigCarLength());
        techParameter.setBigCarLengthUnit(techParamsDTO.getBigCarLengthUnit());
        techParameter.setSmallCarLength(techParamsDTO.getSmallCarLength());
        techParameter.setSmallCarLengthUnit(techParamsDTO.getSmallCarLengthUnit());

        techParameter.setWorkLevel(techParamsDTO.getWorkLevel());

        techParameter.setMainHoistMinSpeed(techParamsDTO.getMainHoistMinSpeed());
        techParameter.setMainHoistMinSpeedUnit(techParamsDTO.getMainHoistMinSpeedUnit());

        techParameter.setMainHoistMaxSpeed(techParamsDTO.getMainHoistMaxSpeed());
        techParameter.setMainHoistMaxSpeedUnit(techParamsDTO.getMainHoistMaxSpeedUnit());

        techParameter.setDeputyHoistMinSpeed(techParamsDTO.getDeputyHoistMinSpeed());
        techParameter.setDeputyHoistMinSpeedUnit(techParamsDTO.getDeputyHoistMinSpeedUnit());

        techParameter.setDeputyHoistMaxSpeed(techParamsDTO.getDeputyHoistMaxSpeed());
        techParameter.setDeputyHoistMaxSpeedUnit(techParamsDTO.getDeputyHoistMaxSpeedUnit());

        techParameter.setBigCarMinSpeed(techParamsDTO.getBigCarMinSpeed());
        techParameter.setBigCarminSpeedUnit(techParamsDTO.getBigCarminSpeedUnit());

        techParameter.setBigCarMaxSpeed(techParamsDTO.getBigCarMaxSpeed());
        techParameter.setBigCarMaxSpeedUnit(techParamsDTO.getBigCarMaxSpeedUnit());

        techParameter.setSmallCarMinSpeed(techParamsDTO.getSmallCarMinSpeed());
        techParameter.setSmallCarminSpeedUnit(techParamsDTO.getSmallCarminSpeedUnit());
        techParameter.setSmallCarMaxSpeed(techParamsDTO.getSmallCarMaxSpeed());
        techParameter.setSmallCarMaxSpeedUnit(techParamsDTO.getSmallCarMaxSpeedUnit());

        techParameter.setMainHoistWeight(techParamsDTO.getMainHoistWeight());
        techParameter.setMainHoistWeightUnit(techParamsDTO.getMainHoistWeightUnit());

        techParameter.setDeputyHoistWeight(techParamsDTO.getDeputyHoistWeight());
        techParameter.setDeputyHoistWeightUnit(techParamsDTO.getDeputyHoistWeightUnit());

        return  techParameter;
    }
}
