package com.lstznkj.groundstation.service.crane;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.lstznkj.groundstation.DTO.ui.DeviceDTO;
import com.lstznkj.groundstation.model.Result;
import com.lstznkj.groundstation.model.crane.Device;
import com.lstznkj.groundstation.model.crane.LifeModel;
import com.lstznkj.groundstation.model.crane.extra.Equipment;
import com.lstznkj.groundstation.repo.crane.DeviceRepository;
import com.lstznkj.groundstation.repo.crane.extra.EquipmentRepository;
import com.lstznkj.groundstation.repo.crane.extra.LifeModelRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class DeviceService {
    @Autowired
    private DeviceRepository deviceRepository;
    private Logger logger= LoggerFactory.getLogger(getClass());
    @Autowired
    private ObjectMapper objectMapper;
    @Autowired
    private EquipmentRepository equipmentRepository;

    @Autowired
    LifeModelRepository lifeModelRepository;
    /**
     *添加设备
     * @param device
     * @return
     */
    public Result<Device> add(DeviceDTO deviceDTO){
        Result<Device> result=new Result<>();
        Device father=null;
        Device device=new Device();
        device.setName(deviceDTO.getName());
        device.setFatherId(deviceDTO.getFatherId());
        device.setRemark(deviceDTO.getRemark());
        try {
            if(device.getFatherId()>0){
                father=deviceRepository.findDeviceById(device.getFatherId());
                device.setFather(father);
                device.setLevel(father.getLevel()+1);
            }
            device=deviceRepository.save(device);

            device=deviceRepository.save(device);
            result.setData(device);

            result.setSuccess(true);
        }catch (Exception ex){
            ex.printStackTrace();
            result.setSuccess(false);
            result.setInfo("未知错误!");
        }
        return result;
    }
public Result<DeviceDTO> updateDevice(DeviceDTO deviceDTO){
    Result<DeviceDTO> result=new Result<>();

    try {
        if(deviceDTO.getFatherId()==deviceDTO.getOldFatherId()){
            //未更改父结构
            Device device=deviceRepository.findDeviceById(deviceDTO.getId());
            device.setName(deviceDTO.getName());
            device.setRemark(deviceDTO.getRemark());

            device=deviceRepository.save(device);

        }
        else {
            Device oldFather=deviceRepository.findDeviceById(deviceDTO.getOldFatherId());
            for(Device d:oldFather.getAllChildren()){
                if(d.getId()==deviceDTO.getId()){
                    oldFather.getAllChildren().remove(d);
                }

            }
            oldFather=deviceRepository.save(oldFather);
            Device newFather=deviceRepository.findDeviceById(deviceDTO.getFatherId());
            Device device=deviceRepository.findDeviceById(deviceDTO.getId());
            device.setFather(newFather);
            newFather.getAllChildren().add(device);
            device=deviceRepository.save(device);
            newFather=deviceRepository.save(newFather);

        }
        result.setSuccess(true);
        result.setData(deviceDTO);
    }
    catch (Exception ex){
        result.setSuccess(false);
        try {
            logger.error("udpate device:"+objectMapper.writeValueAsString(deviceDTO)+ ", error :");
        }
        catch (Exception e1){

        }

        ex.printStackTrace();
    }


    return  result;
}
    /**
     * 递归删除设备及其所有子设备
     * @param device
     */
    private void deleteDeviceAndChildren(Device device){
        if(device.getChildren()!=null){
            for(Device child: device.getChildren()){
                deleteDeviceAndChildren(child);
            }
        }
        Equipment equipment=equipmentRepository.findByDevice_Id(device.getId());
        LifeModel lifeModel=lifeModelRepository.findByDevice_Id(device.getId());
//        equipment.setDevice(null);
//        equipment.setOldDeviceId(device.getId());
//        lifeModel.setDevice(null);
//        lifeModel.setOldDeviceId(device.getId());
//        equipment=equipmentRepository.save(equipment);
//        lifeModel=lifeModelRepository.save(lifeModel);

        if(lifeModel!=null){
            lifeModelRepository.deleteById(lifeModel.getId());
        }
        if(equipment!=null){
            equipmentRepository.deleteById(equipment.getId());
        }


        deviceRepository.delete(device);

    }
    /**
     * 删除指定id的设备
     * @return
     */


    public Result<Device> deleteById(int id){
        Result<Device> result=new Result<>();
        try {
            if(deviceRepository.existsById(id)){
                Device device=deviceRepository.findDeviceById(id);
                deleteDeviceAndChildren(device);
//                if(device.getChildren()!=null){
//                    for(Device child:device.getChildren()){
//                        deviceRepository.delete(child);
//                    }
//                }
//                deviceRepository.deleteById(id);
//                deviceRepository.delete(device);
                result.setSuccess(true);
            }
            else {
                result.setSuccess(false);
                result.setInfo("不存在此设备!");
            }
        }
        catch (Exception ex){
            ex.printStackTrace();
            result.setSuccess(false);
            result.setInfo("未知错误!");
        }
        return result;
    }
    public Result<List<Device>> getAll(){
        Result<List<Device>> result=new Result<>();

        try {
            result.setData(deviceRepository.findAll());
        }
        catch (Exception ex){
            ex.printStackTrace();
            result.setSuccess(false);
            result.setInfo("不存在此设备!");
        }

        return result;
    }

    /***
     * 获取指定设备的所有子设备
     * @param id
     * @return
     */
    public Result<List<Device>> getAllChildren(int id ){
        Result<List<Device>> result=new Result<>();
        List<Device> deviceList=new ArrayList<>();

        Device device=deviceRepository.findDeviceById(id);

        if(device.getChildren()!=null){
            deviceList.addAll(device.getChildren());
            for(Device child : device.getChildren()){
                if(child.getChildren()!=null){
                    deviceList.addAll(getAllChildren(child.getId()).getData());
                }
            }
        }

        result.setData(deviceList);
        return result;
    }

    public Result<Device> getDeviceById(int id ){
        Result<Device> result=new Result<>();
        try {
            result.setData(deviceRepository.findDeviceById(id));
            result.setSuccess(true);
        }
        catch (Exception ex){
            ex.printStackTrace();
            result.setSuccess(false);
            result.setInfo("不存在此设备!");
        }
        return result;
    }

    /**
     * 获取所有根设备（起重机）
     * @return
     */
    public Result<List<Device>> getRootDevices(){
        Result<List<Device>> result=new Result<>();
        try {
            result.setData(deviceRepository.findByLevelIs(0));
            result.setSuccess(true);
        }
        catch (Exception ex){
            result.setSuccess(false);
            result.setInfo("未知错误!!");
        }
        return result;
    }
    public Result<List<Device>> getChildren(int id){
        Result<List<Device>> result=new Result<>();
        try {
            if(deviceRepository.existsById(id)){
                Device device=deviceRepository.findDeviceById(id);
                result.setData(device.getChildren());
                result.setSuccess(true);
            }
            else {
                result.setSuccess(false);
                result.setInfo("不存在此设备!");
            }
        }
        catch (Exception ex){
            result.setSuccess(false);
            result.setInfo("未知错误!!");
        }

        return result;
    }
}
