package com.lstznkj.groundstation.service.crane.extra;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.lstznkj.groundstation.DTO.ui.EquipmentDTO;
import com.lstznkj.groundstation.model.Result;
import com.lstznkj.groundstation.model.crane.Device;
import com.lstznkj.groundstation.model.crane.LifeModel;
import com.lstznkj.groundstation.model.crane.extra.Equipment;
import com.lstznkj.groundstation.repo.crane.DeviceRepository;
import com.lstznkj.groundstation.repo.crane.extra.EquipmentRepository;
import com.lstznkj.groundstation.repo.crane.extra.LifeModelRepository;
import com.lstznkj.groundstation.utils.DateUtil;
import com.lstznkj.groundstation.utils.ObjectUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

@Service
public class EquipmentService {
    @Autowired
    private EquipmentRepository equipmentRepository;
    @Autowired
    private LifeModelRepository lifeModelRepository;
    @Autowired
    private DeviceRepository deviceRepository;
    @Autowired
    private DateUtil dateUtil;
    @Autowired
    private ObjectUtil objectUtil;
    private Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private ObjectMapper objectMapper;

    public Result<EquipmentDTO> update(int equipmentId, EquipmentDTO equipmentDTO) {
        Result<EquipmentDTO> result = new Result<>();
        try {

            logger.debug("start to update equipment:" + objectMapper.writeValueAsString(equipmentDTO));
            Device device = deviceRepository.findDeviceById(equipmentDTO.getDeviceId());
            if(equipmentDTO.getDeviceId()==equipmentDTO.getOldDeviceId()){
                //所属机构未修改
                logger.info("所属机构未修改");
                Equipment nEquipment=transformToEquipment(equipmentDTO);

                Equipment originEquipment=equipmentRepository.findById(equipmentId).get();
                String originUuid= originEquipment.getUuid();
                originEquipment=objectUtil.deepCopy(originEquipment,nEquipment);
                originEquipment.setUuid(originUuid);
                originEquipment.setDevice(device);
                originEquipment=equipmentRepository.save(originEquipment);

               LifeModel nLifeModel=transformToLifeModel(equipmentDTO);

                LifeModel originlifeModel=lifeModelRepository.findByDevice_Id(device.getId());
                int originId=originlifeModel.getId();

                originlifeModel=objectUtil.deepCopy(originlifeModel,nLifeModel);

                originlifeModel.setId(originId);
                originlifeModel.setDevice(device);
                originlifeModel=lifeModelRepository.save(originlifeModel);
                EquipmentDTO nEquipmentDTO=new EquipmentDTO(originEquipment,nLifeModel);
                result.setData(nEquipmentDTO);
                result.setSuccess(true);
            }
            else {
                //所属机构信息被修改
                if(equipmentRepository.existsByDevice_Id(equipmentDTO.getDeviceId())){
                    logger.error("所属机构已修改，并且存在已存在设备信息!");
                    //新修改的  所属机构信息 已经存在设备信息
                    result.setInfo("此机构已经存在设备西信息了！！！");
                    result.setSuccess(false);
                    return  result;
                }
                else {



                    Equipment nEquipment=transformToEquipment(equipmentDTO);

                    Equipment originEquipment=equipmentRepository.findById(equipmentId).get();
                    String originUuid= originEquipment.getUuid();
                    originEquipment=objectUtil.deepCopy(originEquipment,nEquipment);
                    originEquipment.setUuid(originUuid);
                    originEquipment.setDevice(device);
                    originEquipment=equipmentRepository.save(originEquipment);



                    LifeModel nLifeModel=transformToLifeModel(equipmentDTO);

                    LifeModel originlifeModel=lifeModelRepository.findByDevice_Id(equipmentDTO.getOldDeviceId());
                    int originId=originlifeModel.getId();
                    originlifeModel=objectUtil.deepCopy(originlifeModel,nLifeModel);
                    originlifeModel.setId(originId);
                    originlifeModel.setDevice(device);
                    originlifeModel=lifeModelRepository.save(originlifeModel);
                    EquipmentDTO nEquipmentDTO=new EquipmentDTO(originEquipment,nLifeModel);
                    result.setData(nEquipmentDTO);
                    result.setSuccess(true);


                }
            }
//            if (equipmentRepository.existsByDevice_Id(device.getId())) {
//                result.setInfo("此机构已存在设备!!!");
//                result.setSuccess(false);
//                return result;
//            }


//            Equipment equipment = transformToEquipment(equipmentDTO);
//            LifeModel lifeModel = transformToLifeModel(equipmentDTO);
//
//            equipment = equipmentRepository.save(equipment);
//            lifeModel = lifeModelRepository.save(lifeModel);
//            lifeModel.setDevice(device);
//            lifeModel.setEquipmentId(equipment.getId());
//            lifeModel = lifeModelRepository.save(lifeModel);
//
//            equipment.setDevice(device);
//            equipment = equipmentRepository.save(equipment);
//
//            equipmentDTO.setUuid(equipment.getUuid());
//            result.setData(equipmentDTO);
//            result.setSuccess(true);
        } catch (Exception ex) {
            logger.error("update equipment " + equipmentDTO + " error :" + ex.getMessage());
            ex.printStackTrace();
            result.setSuccess(false);
        }

        return result;
    }

    @Transactional
    public Result<EquipmentDTO> deleteById(int id){
        Result<EquipmentDTO>  result=new Result<>();

        try {
            logger.info("start to delete equipment:"+id);
            Equipment equipment=equipmentRepository.findById(id).get();
            LifeModel lifeModel=lifeModelRepository.findByDevice_Id(equipment.getDevice().getId());
            if(lifeModel!=null){
                lifeModel.setOldDeviceId(equipment.getDevice().getId());
                lifeModel.setDevice(null);
                lifeModel=lifeModelRepository.save(lifeModel);
            }

            equipment.setOldDeviceId(equipment.getDevice().getId());
            equipment.setDevice(null);
            equipment=equipmentRepository.save(equipment);

            result.setSuccess(true);
        }
        catch (Exception ex){
            logger.error("delete equipment:"+id+" ,error:"+ex.getMessage());
            result.setSuccess(false);
            ex.printStackTrace();
        }




        return  result;
    }
    public Result<EquipmentDTO> add(EquipmentDTO equipmentDTO) {
        Result<EquipmentDTO> result = new Result<>();

        try {
            logger.debug("start to add equipment:" + objectMapper.writeValueAsString(equipmentDTO));
            Device device = deviceRepository.findDeviceById(equipmentDTO.getDeviceId());

            if (equipmentRepository.existsByDevice_Id(device.getId())) {
                result.setInfo("此机构已存在设备!!!");
                result.setSuccess(false);
                return result;
            }


            Equipment equipment = transformToEquipment(equipmentDTO);
            equipment.setUuid(UUID.randomUUID().toString());

            LifeModel lifeModel = transformToLifeModel(equipmentDTO);

            equipment = equipmentRepository.save(equipment);
            lifeModel = lifeModelRepository.save(lifeModel);

            lifeModel.setDevice(device);
            lifeModel.setEquipmentId(equipment.getId());

            lifeModel = lifeModelRepository.save(lifeModel);

            equipment.setDevice(device);
            equipment = equipmentRepository.save(equipment);

            equipmentDTO.setUuid(equipment.getUuid());
            result.setData(equipmentDTO);
            result.setSuccess(true);
        } catch (Exception ex) {
            logger.error("add equipment " + equipmentDTO + " error :" + ex.getMessage());
            ex.printStackTrace();
            result.setSuccess(false);
        }

        return result;
    }


    /***
     * 获取指定起重机下的所有设备信息
     * @param craneId
     * @return
     */
    public Result<List<EquipmentDTO>> getAllEquipmentsForCrane(int craneId){
        Result<List<EquipmentDTO>> result=new Result<>();
        List<EquipmentDTO> equipmentDTOList = new ArrayList<>();
        try {
            List<Device> deviceList=deviceRepository.findDeviceById(craneId).getAllChildren();
            for(Device device:deviceList){
                Equipment equipment=equipmentRepository.findByDevice_Id(device.getId());

                LifeModel lifeModel=lifeModelRepository.findByDevice_Id(device.getId());
                if(equipment!=null&&lifeModel!=null){
                    EquipmentDTO equipmentDTO = new EquipmentDTO(equipment, lifeModel);
                    equipmentDTOList.add(equipmentDTO);
                }
            }

            result.setData(equipmentDTOList);
            result.setSuccess(true);
        }
        catch (Exception ex){
            ex.printStackTrace();
            result.setSuccess(false);
        }
        return result;
    }

    public Result<List<EquipmentDTO>> getAll() {
        Result<List<EquipmentDTO>> result = new Result<>();

        List<EquipmentDTO> equipmentDTOList = new ArrayList<>();
        try {
            List<Equipment> equipmentList = equipmentRepository.findAll();
            for (Equipment equipment : equipmentList) {
                if(equipment.getDevice()!=null){
                    LifeModel lifeModel = lifeModelRepository.findByDevice_Id(equipment.getDevice().getId());
                    EquipmentDTO equipmentDTO = new EquipmentDTO(equipment, lifeModel);
                    equipmentDTOList.add(equipmentDTO);
                }

            }
            result.setData(equipmentDTOList);
            result.setSuccess(true);
        } catch (Exception ex) {
            logger.error("get all equipmentDTOList error:"+ex.getMessage());
            ex.printStackTrace();
            result.setSuccess(false);
        }


        return result;
    }

    public Equipment findByUuid(String uuid) {
        return equipmentRepository.findByUuid(uuid);
    }

    public void deleteByUuid(String uuid) {
        equipmentRepository.deleteByUuid(uuid);
    }

    private Equipment transformToEquipment(EquipmentDTO equipmentDTO) {
        Equipment equipment = new Equipment();
        equipment.setId(equipmentDTO.getId());
        equipment.setName(equipmentDTO.getName());
        equipment.setSpecification(equipmentDTO.getSpecification());

        equipment.setInstallTime(equipmentDTO.getInstallTime());
        equipment.setPartNo(equipmentDTO.getPartNo());
        equipment.setMaterial(equipmentDTO.getMaterial());
        equipment.setCreateTimestamp(equipmentDTO.getCreateTimestamp());
        equipment.setType(Integer.valueOf(equipmentDTO.getType()));

        equipment.setRemark(equipmentDTO.getRemark());
        return equipment;
    }

    private LifeModel transformToLifeModel(EquipmentDTO equipmentDTO) {
        //待完善

        LifeModel lifeModel = new LifeModel();

        lifeModel.setRunTime(equipmentDTO.getRunTime());
        lifeModel.setActionCount(equipmentDTO.getActionCount());
        lifeModel.setBrakingCount(equipmentDTO.getBrakingCount());
        lifeModel.setUpdateCycle(equipmentDTO.getUpdateCycle());
        lifeModel.setUpdateCycleUnit(equipmentDTO.getUpdateCycleUnit());

        return lifeModel;
    }
}
